import java.awt.*;

/**
 * An abstract class that represents a DancingTurtle. This class should be extended to create concrete dancers
 */
public abstract class DancingTurtle
{
	//Constants for tracking direction. Each represents a vector along which we
	public static final Point NORTH = new Point(0,-1);
	public static final Point EAST = new Point(1,0);
	public static final Point SOUTH = new Point(0,1);
	public static final Point WEST = new Point(-1,0);
	
	//attributes of the DancingTurtle
	protected Point direction;
	protected int x;
	protected int y;
	protected Color color;

	//dance floor boundaries (move legal)
	private final Dimension MIN_BOUND = new Dimension(0+15, 0+15); 
	private final Dimension MAX_BOUND = new Dimension(DanceFloor.FLOOR_WIDTH-55, DanceFloor.FLOOR_HEIGHT-55);
	
	/**
	 * Abstract constructor for creating a new DancingTurtle at the given location with the given facing and color.
	 * This constructor cannot be called directly, but can be called by children to ease variable assignment.
	 * @param x the turtle's x location
	 * @param y the turtle's y location
	 * @param direction the turtle's direction (should be a class constant: NORTH, EAST, SOUTH, or WEST)
	 * @param color the turtle's color
	 */
	public DancingTurtle(int x, int y, Point direction, Color color)
	{
		this.x = x;
		this.y = y;
		this.direction = direction;
		this.color = color;
	}

	/**
	 * An abstract method (that must be overridden) that causes the turtle to perform a single dance step.
	 * The details of what that step IS should be defined by the concrete child class.
	 */
	public abstract void step();
	
	/**
	 * A method that spins the turtle 90degrees to the right (clockwise) 
	 * @return whether or not the turtle successfully turned.
	 */
	public boolean spinRight()
	{
	    //hard-code new directions to ease in comparison. Our direction should always be a class constant
		if(this.direction == NORTH)
		{
			this.direction = EAST;
		}
		else if(this.direction == EAST)
		{
			this.direction = SOUTH;
		}
		else if(this.direction == SOUTH)
		{
			this.direction = WEST;
		}
		else if(this.direction == WEST)
		{
			this.direction = NORTH;
		}
		return true;
	}

	/**
	 * A method that spins the turtle 90degrees to the left (counter-clockwise) 
	 * @return whether or not the turtle successfully turned.
	 */
	public boolean spinLeft()
	{
	    //hard-code new directions to ease in comparison. Our direction should always be a class constant
		if(this.direction == NORTH)
		{
			this.direction = WEST;
		}
		else if(this.direction == WEST)
		{
			this.direction = SOUTH;
		}
		else if(this.direction == SOUTH)
		{
			this.direction = EAST;
		}
		else if(this.direction == EAST)
		{
			this.direction = NORTH;
		}
		return true;
	}

	
	/**
	 * Moves the turtle along the line it is facing. Note that the turtle can move backwards.
	 * The turtle will only move if it stays within the bounds of the dance floor.
	 * @param dist the number of pixels the turtle should move. A negative number causes the turtle to step backwards.
	 * @return whether or not the turtle successfully moved.
	 */
	public boolean moveForward(int dist)
	{
		int newX = x + dist*this.direction.x; //where we want to move to (direction vector * distance)
		int newY = y + dist*this.direction.y;
	
		//move if we can!
		if(newX >= MIN_BOUND.width && newX <= MAX_BOUND.width && newY >= MIN_BOUND.height && newY <= MAX_BOUND.height)
		{
			x = newX;
			y = newY;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	//relative to facing: positive distance is to the right, negative is to the left
	/**
	 * Moves the turtle sideways relative to its facing A positive parameter means it moves to the right, 
	 * a negative parameter means it moves to the left.
	 * The turtle will only move if it stays within the bounds of the dance floor.
	 * @param dist the number of pixels the turtle should move. A negative number causes the turtle to move right, a negative to move left.
	 * @return whether or not the turtle successfully moved.
	 */
	public boolean moveSide(int dist)
	{
		int newX = x - dist*this.direction.y; //where we want to move to; inverted direction
		int newY = y + dist*this.direction.x;

		//move if we can!
		if(newX >= MIN_BOUND.width && newX <= MAX_BOUND.width && newY >= MIN_BOUND.height && newY <= MAX_BOUND.height)
		{
			x = newX;
			y = newY;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Draws the turtle on the given graphics context. The turtle is currently pretty simple.
	 * @param g a Graphics object on which to draw the turtle
	 */
	public void draw(Graphics g)
	{
		g.setColor(color);

		g.fillOval(x,y,40,40); //draw the body

		//draw the head
		if(this.direction == NORTH)
		{
			g.fillOval(x+10, y-15, 20, 20);
		}
		else if(this.direction == WEST)
		{
			g.fillOval(x-15, y+10, 20, 20);
		}
		else if(this.direction == SOUTH)
		{
			g.fillOval(x+10, y+35, 20, 20);
		}
		else if(this.direction == EAST)
		{
			g.fillOval(x+35, y+10, 20, 20);
		}
	}
	
}
